@@ -1,6 +1,8 @@
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
+ * Copyright (C) 2002-2017 Németh László
+ *
  * The contents of this file are subject to the Mozilla Public License Version
  * 1.1 (the "License"); you may not use this file except in compliance with
  * the License. You may obtain a copy of the License at
@@ -11,12 +13,7 @@
  * for the specific language governing rights and limitations under the
  * License.
  *
- * The Original Code is Hunspell, based on MySpell.
- *
- * The Initial Developers of the Original Code are
- * Kevin Hendricks (MySpell) and Németh László (Hunspell).
- * Portions created by the Initial Developers are Copyright (C) 2002-2005
- * the Initial Developers. All Rights Reserved.
+ * Hunspell is based on MySpell which is Copyright (C) 2002 Kevin Hendricks.
  *
  * Contributor(s): David Einstein, Davide Prina, Giuseppe Modugno,
  * Gianluca Turconi, Simon Brouwer, Noll János, Bíró Árpád,
@@ -86,7 +83,7 @@
 
 #define MAXWORDUTF8LEN (MAXWORDLEN * 3)
 
-class HunspellImpl
+class LIBHUNSPELL_DLL_EXPORTED HunspellImpl
 {
 public:
   HunspellImpl(const char* affpath, const char* dpath, const char* key);
@@ -109,7 +106,8 @@ public:
   int add_with_affix(const std::string& word, const std::string& example);
   int remove(const std::string& word);
   const std::string& get_version() const;
-
+  struct cs_info* get_csconv();
+  std::vector<char> dic_encoding_vec;
 
 private:
   AffixMgr* pAMgr;
@@ -180,6 +178,9 @@ HunspellImpl::HunspellImpl(const char* affpath, const char* dpath, const char* k
   complexprefixes = pAMgr->get_complexprefixes();
   wordbreak = pAMgr->get_breaktable();
 
+  dic_encoding_vec.resize(encoding.size()+1);
+  strcpy(&dic_encoding_vec[0], encoding.c_str());
+
   /* and finally set up the suggestion manager */
   pSMgr = new SuggestMgr(try_string, MAXSUGGESTION, pAMgr);
   if (try_string)
@@ -237,8 +238,8 @@ size_t HunspellImpl::cleanword2(std::string& dest,
   const char* q = src.c_str();
 
   // first skip over any leading blanks
-  while ((*q != '\0') && (*q == ' '))
-    q++;
+  while (*q == ' ')
+    ++q;
 
   // now strip off any trailing periods (recording their presence)
   *pabbrev = 0;
@@ -274,8 +275,8 @@ void HunspellImpl::cleanword(std::string& dest,
   int firstcap = 0;
 
   // first skip over any leading blanks
-  while ((*q != '\0') && (*q == ' '))
-    q++;
+  while (*q == ' ')
+    ++q;
 
   // now strip off any trailing periods (recording their presence)
   *pabbrev = 0;
@@ -1246,7 +1247,11 @@ std::vector<std::string> HunspellImpl::stem(const std::string& word) {
   return stem(analyze(word));
 }
 
-const std::string& Hunspell::get_wordchars() const {
+const char* Hunspell::get_wordchars() const {
+  return m_Impl->get_wordchars().c_str();
+}
+
+const std::string& Hunspell::get_wordchars_cpp() const {
   return m_Impl->get_wordchars();
 }
 
@@ -1323,7 +1328,11 @@ int HunspellImpl::remove(const std::string& word) {
   return 0;
 }
 
-const std::string& Hunspell::get_version() const {
+const char* Hunspell::get_version() const {
+  return m_Impl->get_version().c_str();
+}
+
+const std::string& Hunspell::get_version_cpp() const {
   return m_Impl->get_version();
 }
 
@@ -1331,6 +1340,14 @@ const std::string& HunspellImpl::get_version() const {
   return pAMgr->get_version();
 }
 
+struct cs_info* HunspellImpl::get_csconv() {
+  return csconv;
+}
+
+struct cs_info* Hunspell::get_csconv() {
+  return m_Impl->get_csconv();
+}
+
 void HunspellImpl::cat_result(std::string& result, const std::string& st) {
   if (!st.empty()) {
     if (!result.empty())
@@ -1644,7 +1661,7 @@ std::string HunspellImpl::get_xml_par(const char* par) {
   if (end == '>')
     end = '<';
   else if (end != '\'' && end != '"')
-    return 0;  // bad XML
+    return dest;  // bad XML
   for (par++; *par != '\0' && *par != end; ++par) {
     dest.push_back(*par);
   }
@@ -1665,6 +1682,16 @@ bool Hunspell::input_conv(const std::string& word, std::string& dest) {
   return m_Impl->input_conv(word, dest);
 }
 
+int Hunspell::input_conv(const char* word, char* dest, size_t destsize) {
+  std::string d;
+  bool ret = input_conv(word, d);
+  if (ret && d.size() < destsize) {
+    strncpy(dest, d.c_str(), destsize);
+    return 1;
+  }
+  return 0;
+}
+
 bool HunspellImpl::input_conv(const std::string& word, std::string& dest) {
   RepList* rl = pAMgr ? pAMgr->get_iconvtable() : NULL;
   if (rl) {
@@ -1677,14 +1704,17 @@ bool HunspellImpl::input_conv(const std::string& word, std::string& dest) {
 // return the beginning of the element (attr == NULL) or the attribute
 const char* HunspellImpl::get_xml_pos(const char* s, const char* attr) {
   const char* end = strchr(s, '>');
-  const char* p = s;
   if (attr == NULL)
     return end;
-  do {
+  const char* p = s;
+  while (1) {
     p = strstr(p, attr);
     if (!p || p >= end)
       return 0;
-  } while (*(p - 1) != ' ' && *(p - 1) != '\n');
+    if (*(p - 1) == ' ' || *(p - 1) == '\n')
+      break;
+    p += strlen(attr);
+  }
   return p + strlen(attr);
 }
 
@@ -1820,8 +1850,8 @@ int Hunspell::suffix_suggest(char*** slst, const char* root_word) {
   return munge_vector(slst, stems);
 }
 
-const char* Hunspell::get_dic_encoding() const {
-  return Hunspell_get_dic_encoding((Hunhandle*)(this));
+char* Hunspell::get_dic_encoding() {
+  return &(m_Impl->dic_encoding_vec[0]);
 }
 
 int Hunspell::stem(char*** slst, char** desc, int n) {
@@ -1866,8 +1896,8 @@ int Hunspell_spell(Hunhandle* pHunspell, const char* word) {
   return reinterpret_cast<Hunspell*>(pHunspell)->spell(std::string(word));
 }
 
-const char* Hunspell_get_dic_encoding(Hunhandle* pHunspell) {
-  return (reinterpret_cast<Hunspell*>(pHunspell)->get_dict_encoding()).c_str();
+char* Hunspell_get_dic_encoding(Hunhandle* pHunspell) {
+  return reinterpret_cast<Hunspell*>(pHunspell)->get_dic_encoding();
 }
 
 int Hunspell_suggest(Hunhandle* pHunspell, char*** slst, const char* word) {
